@@ -10,7 +10,14 @@
 
 #include "inotify.h"
 
+#include "event_queue.h"
+
 #define ALL_MASK 0xffffffff
+#define EVENTQ_SIZE 128
+
+struct inotify_event *eventq[EVENTQ_SIZE];
+int eventq_head = 0;
+int eventq_tail = 0;
 
 int open_dev ()
 {
@@ -52,13 +59,9 @@ void print_mask(int mask)
 	{
 		printf("ATTRIB ");
 	}
-	if (mask & IN_CLOSE_WRITE)
+	if (mask & IN_CLOSE)
 	{
-		printf("CLOSE_WRITE ");
-	}
-	if (mask & IN_CLOSE_NOWRITE)
-	{
-		printf("CLOSE_NOWRITE ");
+		printf("CLOSE ");
 	}
 	if (mask & IN_OPEN)
 	{
@@ -105,21 +108,63 @@ void print_mask(int mask)
 		printf("IGNORED" );
 	}
 
+	printf("0x%08x", mask);
+
 	printf("\n");
 }
 
-void print_event (struct inotify_event event)
+void print_event (struct inotify_event *event)
 {
-	printf ("EVENT ON WD=%d\n", event.wd);
-	print_mask (event.mask);
-	printf ("FILENAME=%s\n", event.filename);
+	printf ("EVENT ON WD=%d\n", event->wd);
+	print_mask (event->mask);
+	if (event->len)
+		printf ("FILENAME=%s\n", event->filename);
+	printf("\n");
 }
 
-int read_event (int fd, struct inotify_event *event)
+void print_events (queue_t q)
 {
-	event->wd = 0xdeadbeef;
-	event->mask = 0xdeadbeef;
-	return read (fd, event, sizeof(struct inotify_event));
+	struct inotify_event *event;
+	while (!queue_empty (q))
+	{
+		event = queue_front (q);
+		queue_dequeue (q);
+		print_event (event);
+		free (event);
+	}
+}
+
+int read_events (queue_t q, int fd)
+{
+	char buffer[16384];
+	size_t buffer_i;
+	struct inotify_event *pevent, *event;
+	ssize_t r;
+	size_t event_size;
+	int count = 0;
+
+	r = read (fd, buffer, 16384);
+
+	if (r <= 0)
+		return r;
+
+	buffer_i = 0;
+	printf("read = %d\n", r);
+
+	printf("sizeof inotify_event = %d\n", sizeof(struct inotify_event));
+	while (buffer_i < r) {
+		/* Parse events and queue them ! */
+		pevent = (struct inotify_event *)&buffer[buffer_i];
+		event_size = sizeof(struct inotify_event) + pevent->len;
+		printf("pevent->len = %d\n", pevent->len);
+		event = malloc(event_size);
+		memmove(event, pevent, event_size);
+		queue_enqueue(event, q);
+		buffer_i += event_size;
+		count++;
+	}
+
+	return count;
 }
 
 int event_check (int fd)
@@ -128,8 +173,8 @@ int event_check (int fd)
 	int r;
 	fd_set rfds;
 
-	timeout.tv_sec = 0;
-	timeout.tv_usec = 10000;
+	timeout.tv_sec = 4;
+	timeout.tv_usec = 0;
 
 	FD_ZERO(&rfds);
 	FD_SET(fd, &rfds);
@@ -139,28 +184,27 @@ int event_check (int fd)
 	return r;
 }
 
-int read_and_print_events (int fd)
+int read_and_print_events (queue_t q, int fd)
 {
-	struct inotify_event event;
-
 	while (1)
 	{
+		if (!queue_empty(q))
+			print_events (q);
+
 		if (event_check(fd) > 0) {
 			int r;
-			r = read_event(fd, &event);
 
-			if (r > 0) {
-				print_event (event);
-			} else {
+			r = read_events (q, fd);
+
+			if (r < 0)
 				break;
-			}
 		}
 	}
 
 	return 0;
 }
 
-int watch_dir (int fd, const char *dirname, unsigned long mask)
+int watch_dir (int fd, const char *dirname, long int mask)
 {
 	struct inotify_watch_request iwr;
 	iwr.dirname = strdup (dirname);
@@ -170,7 +214,7 @@ int watch_dir (int fd, const char *dirname, unsigned long mask)
 	wd = ioctl(fd, INOTIFY_WATCH, &iwr);
 
 	if (wd < 0) {
-		printf("ioctl(fd, INOTIFY_WATCH, {%s, %u}) =", iwr.dirname, iwr.mask);
+		printf("ioctl(fd, INOTIFY_WATCH, {%s, %d}) = %d", iwr.dirname, iwr.mask, wd);
 		fflush(stdout);
 		perror (" ");
 	}
@@ -193,4 +237,3 @@ int ignore_wd (int fd, int wd)
 
 	return r;
 }
-
